Package: purrr
Title: Functional Programming Tools
Version: 1.2.2
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Lionel", "Henry", , "lionel@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "https://ror.org/03wc8by49"))
  )
Description: A complete and consistent functional programming toolkit for
    R.
License: MIT + file LICENSE
URL: https://purrr.tidyverse.org/, https://github.com/tidyverse/purrr
BugReports: https://github.com/tidyverse/purrr/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.6.1), lifecycle (>= 1.0.3), magrittr (>= 1.5.0),
        rlang (>= 1.1.1), vctrs (>= 0.6.3)
Suggests: carrier (>= 0.3.0), covr, dplyr (>= 0.7.8), httr, knitr,
        lubridate, mirai (>= 2.5.1), rmarkdown, testthat (>= 3.0.0),
        tibble, tidyselect
LinkingTo: cli
VignetteBuilder: knitr
Biarch: true
Config/build/compilation-database: true
Config/Needs/website: tidyverse/tidytemplate, tidyr
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-10 10:15:54 UTC; lionel
Author: Hadley Wickham [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Lionel Henry [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-04-10 17:00:09 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-10 18:05:41 UTC; unix
Archs: purrr.so.dSYM
