#' @export
rlang::set_names

#' @export
rlang::exec

#' @export
rlang::zap

#' @export
rlang::`%||%`

#' @export
rlang::done

#' @export
rlang::rep_along

# Predicates ---------------------------------------------------

#' @export
rlang::is_bare_list

#' @export
rlang::is_bare_atomic

#' @export
rlang::is_bare_vector

#' @export
rlang::is_bare_double

#' @export
rlang::is_bare_integer

#' @export
rlang::is_bare_numeric

#' @export
rlang::is_bare_character

#' @export
rlang::is_bare_logical

#' @export
rlang::is_list

#' @export
rlang::is_atomic

#' @export
rlang::is_vector

#' @export
rlang::is_integer

#' @export
rlang::is_double

#' @export
rlang::is_character

#' @export
rlang::is_logical

#' @export
rlang::is_null

#' @export
rlang::is_function

#' @export
rlang::is_scalar_list

#' @export
rlang::is_scalar_atomic

#' @export
rlang::is_scalar_vector

#' @export
rlang::is_scalar_double

#' @export
rlang::is_scalar_character

#' @export
rlang::is_scalar_logical

#' @export
rlang::is_scalar_integer

#' @export
rlang::is_empty

#' @export
rlang::is_formula
